.box {
    padding-left: 36px;
    padding-right: 48px;
    display: flex;
    justify-content: space-around;
    height: 100px;
    border-radius: 12px;
    background: #f7fafe;
}

.box-left {
    position: relative;
    width: 124px;
}
.box-right {
    position: relative;
    width: 36px;
}

.btn {
    padding: 0;
    margin: auto;
    position: absolute;
    left: 50%;
    top: 50%;
    bottom: 50%;
    right: 50%;
    font-size: 36px;
    line-height: 36px;
}

.up {
    transform: translateX(-18px) translateY(-28px);
}

.down {
    transform: translateX(-18px) translateY(28px);
}

.left {
    transform: translateX(-54px);
}

.right {
    transform: translateX(18px);
}

.input {
    width: 100%;
}
